# ############################################################################### #
# Autoreduction Repository : https://github.com/autoreduction/autoreduce
#
# Copyright &copy; 2021 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""Module for the search runs page model."""
from typing import List

from django.urls import reverse
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.by import By

from autoreduce_frontend.selenium_tests.pages.component_mixins.footer_mixin import FooterMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.navbar_mixin import NavbarMixin
from autoreduce_frontend.selenium_tests.pages.page import Page
from autoreduce_frontend.selenium_tests.pages.run_summary_page import RunSummaryPage


class SearchPage(Page, NavbarMixin, FooterMixin):

    def __init__(self, driver, instrument):
        super().__init__(driver)
        self.instrument = instrument

    @staticmethod
    def url_path():
        """
        Return the path section of the overview page url
        :return: (str) Path section of the page url
        """
        return reverse("search")

    @property
    def run_number_text_area(self) -> WebElement:
        """Return the run-number text area."""
        return self.driver.find_element(By.ID, "id_run_number")

    @property
    def run_instrument_dropdown(self) -> WebElement:
        """Return the instrument dropdown selection."""
        dropdown = Select(self.driver.find_element(By.ID, 'id_instrument'))
        return dropdown

    @property
    def run_description_text_area(self) -> WebElement:
        """Return the run-description text area."""
        return self.driver.find_element(By.ID, "id_run_description")

    @property
    def created_min_date(self) -> WebElement:
        """Return the first (beginning of Date range) date picker."""
        created_min = self.driver.find_element(By.ID, "id_created_0")
        return created_min

    @property
    def created_max_date(self) -> WebElement:
        """Return the second (end of Date range) date picker."""
        created_max = self.driver.find_element(By.ID, "id_created_1")
        return created_max

    @property
    def run_description_contains(self) -> WebElement:
        """Return the "contains" radio button for run description."""
        contains_radio_button = self.driver.find_element(By.XPATH, "//*[@id='contains']")
        return contains_radio_button

    @property
    def run_description_exact(self) -> WebElement:
        """Return the "exact" radio button for run description."""
        exact_radio_button = self.driver.find_element(By.XPATH, "//*[@id='exact']")
        return exact_radio_button

    @property
    def reference_number_text_area(self) -> WebElement:
        """Return the reference number text area on Experiments tab."""
        return self.driver.find_element(By.ID, "id_reference_number")

    def click_runs_tab(self) -> None:
        """
        Clicks run tab to show Run search form
        """
        runs_tab = self.driver.find_element(By.ID, "pills-runs-tab")
        runs_tab.click()

    def click_experiments_tab(self) -> None:
        """
        Clicks experiments tab to show Experiments search form
        """
        experiments_tab = self.driver.find_element(By.ID, "pills-experiments-tab")
        experiments_tab.click()

    def alert_runs_message_text(self) -> str:
        """
        Return the text of the alert message element with the id
        'run_message'.
        """
        return self.driver.find_element(By.ID, "run_message").text.strip()

    def alert_experiments_message_text(self) -> str:
        """
        Return the text of the alert message element with the id
        'experiment_message'.
        """
        return self.driver.find_element(By.ID, "experiment_message").text.strip()

    def get_run_numbers_from_table(self) -> List[str]:
        """
        Return the list of run numbers visible on the current table of the
        instrument summary page.
        """
        return [run.text.split(" - ")[0] for run in self.driver.find_elements(By.CLASS_NAME, "run-num-links")]

    def get_experiments_from_table(self) -> List[str]:
        """
        Return the list of run numbers visible on the current table of the
        instrument summary page.
        """
        return [run.text.split(" - ")[0] for run in self.driver.find_elements(By.CLASS_NAME, "experiment-num-links")]

    def click_search_button(self) -> None:
        """Click the `Search` button."""
        search_btn = self.driver.find_element(By.ID, "search-button")
        search_btn.click()

    def click_run(self, run_number: int, version: int = 0) -> RunSummaryPage:
        """
        Click a run and return the page object of its run summary.
        Args:
            run_number: Run number of the link to click.
            version: Version of the run to click.
        Returns:
            The page object of the opened run summary.
        """
        runs = self.driver.find_elements(By.XPATH, "//div[@class='table-container']/table/tbody/tr/td/a")
        run_string = f"{run_number} - {version}" if version else f"{run_number}"
        for run in runs:
            if run.text == run_string:
                run.click()
                return RunSummaryPage(self.driver, self.instrument, run_number, version)

        raise NoSuchElementException



# ############################################################################### #
# Autoreduction Repository : https://github.com/autoreduction/autoreduce
#
# Copyright &copy; 2020 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
# pylint: disable=invalid-name,redefined-builtin
from django.conf import settings
from django.conf.urls import include
from django.contrib import admin
from django.urls import path, register_converter

from autoreduce_frontend.reduction_viewer.views import (accessibility_statement, experiment_summary, graph, help, index,
                                                        logout, overview, stats, search)


class NegativeIntConverter:
    regex = r'-?\d+'

    @staticmethod
    def to_python(value):
        """Return the value as a Python object."""
        return int(value)

    @staticmethod
    def to_url(value):
        """Return the value as a URL string."""
        return f'{value}'


register_converter(NegativeIntConverter, 'negint')

urlpatterns = [
    # ===========================MISC================================= #
    path('', index.index, name='index'),
    path('admin/', admin.site.urls),
    path('logout/', logout.logout, name='logout'),
    path('help/', help.help, name='help'),
    path('accessibility_statement/', accessibility_statement.accessibility_statement, name='accessibility_statement'),

    # ===========================RUNS================================= #
    path('overview/', overview.overview, name='overview'),
    path('runs/', include('reduction_viewer.urls')),

    # ===========================EXPERIMENT========================== #
    path('experiment/<negint:reference_number>/', experiment_summary.experiment_summary, name='experiment_summary'),

    # ===========================SCRIPTS============================= #
    path('graph/', graph.graph_home, name="graph"),
    path('graph/<str:instrument_name>', graph.graph_instrument, name="graph_instrument"),
    path('stats/', stats.stats, name="stats"),

    # =======================GENERATE TOKEN========================== #
    path('tokens/', include('generate_token.urls')),
    path('search/', search.search, name='search'),
]

if settings.DEBUG_TOOLBAR_AVAILABLE:
    urlpatterns = [
        path('__debug__/', include("debug_toolbar.urls")),
    ] + urlpatterns



# ############################################################################### #
# Autoreduction Repository : https://github.com/autoreduction/autoreduce
#
# Copyright &copy; 2021 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""
Module for the job queue page model
"""
from typing import Optional, Union, List

from functools import partial
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.by import By
from django.urls.base import reverse
from autoreduce_frontend.selenium_tests.pages.component_mixins.footer_mixin import FooterMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.navbar_mixin import NavbarMixin
from autoreduce_frontend.selenium_tests.pages.page import Page


class JobQueuePage(Page, NavbarMixin, FooterMixin):

    @staticmethod
    def url_path() -> str:
        """
        Return the path section of the job queue url
        :return: (str) path section of job queue url
        """
        return reverse("runs:queue")

    def get_run_numbers_from_table(self) -> List[str]:
        """
        Return a list of run numbers from the table
        :return: (List) list of string run numbers from table
        """
        return [run.text for run in self.driver.find_elements(By.CLASS_NAME, "run-link")]

    def get_status_from_run(self,
                            run_number: Union[str, int],
                            last_run_number: Optional[Union[str, int]] = None) -> str:
        """
        Given a run number return the status of the run as shown in the table
        :param run_number: (str/int) The run number
        :return: (status) The status as a string
        """
        return self.driver.find_element(
            By.ID, f"status-{run_number}-{run_number if not last_run_number else last_run_number}").text

    def _do_run_button(self, url):

        def run_button_clicked_successfully(button, url, driver):
            button.click()
            return url in driver.current_url

        button = self.driver.find_element(By.CSS_SELECTOR, f'[href*="{url}"]')
        WebDriverWait(self.driver, 10).until(partial(run_button_clicked_successfully, button, url))

    def click_run(self, run_number: Union[str, int]) -> None:
        """
        Click the run number in the table
        :param run_number: The run number to click
        """
        url = reverse("runs:summary", kwargs={"instrument_name": "TESTINSTRUMENT", "run_number": run_number})
        self._do_run_button(url)

    def click_batch_run(self, primary_key: Union[str, int]) -> None:
        """
        Click the batch run link in the table

        :param primary_key: The primary key of the batch run
        """
        url = reverse("runs:batch_summary", kwargs={"instrument_name": "TESTINSTRUMENT", "pk": primary_key})
        self._do_run_button(url)



# ############################################################################### #
# Autoreduction Repository : https://github.com/autoreduction/autoreduce
#
# Copyright &copy; 2021 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""Module for the instrument summary page model."""
from typing import List

from django.urls import reverse
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.by import By

from autoreduce_frontend.selenium_tests.pages.component_mixins.footer_mixin import FooterMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.navbar_mixin import NavbarMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.tour_mixin import TourMixin
from autoreduce_frontend.selenium_tests.pages.page import Page
from autoreduce_frontend.selenium_tests.pages.run_summary_page import RunSummaryPage


class RunsListPage(Page, NavbarMixin, FooterMixin, TourMixin):
    """Page model class for instrument summary page."""

    def __init__(self, driver, instrument):
        super().__init__(driver)
        self.instrument = instrument

    def url_path(self):
        """Return the path section of the instrument url."""
        return reverse("runs:list", kwargs={"instrument": self.instrument})

    def get_run_numbers_from_table(self) -> List[str]:
        """
        Return the list of run numbers visible on the current table of the
        instrument summary page.
        """
        return [run.text.split(" - ")[0] for run in self.driver.find_elements(By.CLASS_NAME, "run-num-links")]

    def get_experiments_from_table(self) -> List[str]:
        """
        Return the list of run numbers visible on the current table of the
        instrument summary page.
        """
        return [run.text.split(" - ")[0] for run in self.driver.find_elements(By.CLASS_NAME, "experiment-num-links")]

    def get_created_from_table(self) -> List[str]:
        """
        Return the list of created dates visible on the current table of the
        instrument summary page.
        """
        return [run.text.split(" - ")[0] for run in self.driver.find_elements(By.CLASS_NAME, "created-dates")]

    def get_status_from_table(self) -> List[str]:
        """
        Return the list of status' visible on the current table of the
        instrument summary page.
        """
        return [run.text.split(" - ")[0] for run in self.driver.find_elements(By.CLASS_NAME, "run-status")]

    def click_run(self, run_number: int, version: int = 0) -> RunSummaryPage:
        """
        Click a run and return the page object of its run summary.

        Args:
            run_number: Run number of the link to click.

            version: Version of the run to click.

        Returns:
            The page object of the opened run summary.
        """
        runs = self.driver.find_elements(By.XPATH, "//div[@class='table-container']/table/tbody/tr/td/a")
        run_string = f"{run_number} - {version}" if version else f"{run_number}"
        for run in runs:
            if run.text == run_string:
                run.click()
                return RunSummaryPage(self.driver, self.instrument, run_number, version)

        raise NoSuchElementException

    def variables_alert_message_text(self) -> str:
        """
        Return the text of the alert message element with the id
        'variable_alert_message'.
        """
        return self.driver.find_element(By.ID, "variables_alert_message").text.strip()

    def get_top_run(self) -> WebElement:
        """Return the element with the id 'top-run-number'."""
        return self.driver.find_element(By.XPATH, "//div[@class='table-container']/table/tbody/tr[1]/td[1]/a")

    def click_btn_by_title(self, title: str) -> None:
        """
        Click the button matching the given title.

        Note:
            This method is being used to navigate instrument summary pages as
            there is no `find_element_by_title` method and the instrument
            summary navigation buttons have no id attributes.
        """
        btns = self.driver.find_elements(By.TAG_NAME, "button")

        for btn in btns:
            if btn.get_attribute("title") == title:
                btn.click()
                break
        else:
            raise NoSuchElementException

    def click_next_page_button(self) -> None:
        """
        Click next page pagination button
        """
        btn = self.driver.find_element(By.XPATH, "//li[@class='next page-item']/a")
        btn.click()

    def click_prev_page_button(self) -> None:
        """
        Click previous page pagination button
        """
        btn = self.driver.find_element(By.XPATH, "//li[@class='prev page-item']/a")
        btn.click()

    def update_filter(self, filter_name, value):
        """
        Select a valid filter option.

        Args:
            filter_name: The name of the filter type being updated.

            value: The new value of the given filter.
        """
        Select(self.driver.find_element(By.ID, filter_name)).select_by_visible_text(value)

    def click_apply_filters(self) -> None:
        """Click the `Apply filters` button."""
        btn = self.driver.find_element(By.ID, "apply_filters")
        btn.click()

    @property
    def top_alert_message_text(self):
        """Finds and returns the alert message shown at the top of the runs list page"""
        return self.driver.find_element(By.ID, "top-alert-message").text



# ############################################################################### #
# Autoreduction Repository : https://github.com/autoreduction/autoreduce
#
# Copyright &copy; 2021 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""
Module for the help summary page model
"""

from django.urls.base import reverse
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.by import By

from autoreduce_frontend.selenium_tests.pages.component_mixins.footer_mixin import FooterMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.navbar_mixin import NavbarMixin
from autoreduce_frontend.selenium_tests.pages.page import Page


class GenerateTokenFormPage(Page, NavbarMixin, FooterMixin):
    """
    Page model class for help page
    """

    @staticmethod
    def url_path() -> str:
        """
        Return the current URL of the page.
        :return: (str) the url path
        """
        return reverse("token:generate")

    def click_generate_token(self) -> WebElement:
        """Clicks generate in the token generation page"""
        return self.driver.find_element(By.ID, "generate-token-submit").click()

    def generate_form_users(self) -> Select:
        """Grabs the users Select field in the generate new token form"""
        return Select(self.driver.find_element(By.ID, "id_user"))



from django.urls import path
from autoreduce_frontend.autoreduce_webapp.view_utils import login_and_uows_valid

from autoreduce_frontend.generate_token import views

app_name = "token"

urlpatterns = [
    path("", login_and_uows_valid(views.ShowToken.as_view()), name="list"),
    path("generate", login_and_uows_valid(views.GenerateTokenFormView.as_view()), name="generate"),
    path("delete/<str:pk>", login_and_uows_valid(views.DeleteToken.as_view()), name="delete"),
]



# ############################################################################### #
# Autoreduction Repository : https://github.com/autoreduction/autoreduce
#
# Copyright &copy; 2021 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""Module for the run summary page model."""
from functools import partial
from typing import List

from django.urls.base import reverse
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.by import By

from autoreduce_frontend.selenium_tests.pages.component_mixins.footer_mixin import FooterMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.navbar_mixin import NavbarMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.rerun_form_mixin import RerunFormMixin
from autoreduce_frontend.selenium_tests.pages.component_mixins.tour_mixin import TourMixin
from autoreduce_frontend.selenium_tests.pages.page import Page


class RunSummaryPage(Page, RerunFormMixin, NavbarMixin, FooterMixin, TourMixin):
    """Page model class for run summary page."""

    def __init__(self, driver, instrument, run_number, version, batch_run=False):
        super().__init__(driver)
        self.instrument = instrument
        self.run_number = run_number
        self.version = version
        self.batch_run = batch_run

    def url_path(self) -> str:
        """Return the current URL of the page."""
        if not self.batch_run:
            return reverse("runs:summary",
                           kwargs={
                               "instrument_name": self.instrument,
                               "run_number": self.run_number,
                               "run_version": self.version
                           })
        else:
            return reverse("runs:batch_summary",
                           kwargs={
                               "instrument_name": self.instrument,
                               "pk": self.run_number,
                               "run_version": self.version
                           })

    @property
    def reduction_job_panel(self) -> WebElement:
        """Return the run summary panel."""
        return self.driver.find_element(By.ID, "reduction_job_panel")

    @property
    def rerun_form(self) -> WebElement:
        """Return the rerun form."""
        return self.driver.find_element(By.ID, "rerun_form")

    @property
    def toggle_button(self) -> WebElement:
        """Return the toggle button."""
        return self.driver.find_element(By.ID, "toggle_form")

    @property
    def reset_to_initial_values(self) -> WebElement:
        """Return the `Reset to original script and values` button."""
        return self.driver.find_element(By.ID, "resetValues")

    @property
    def reset_to_current_values(self) -> WebElement:
        """
        Return the `Reset to values in the current reduce_vars script` button.
        """
        return self.driver.find_element(By.ID, "currentScript")

    @property
    def warning_message(self) -> WebElement:
        """Return the 'warning_message' box."""
        return self.driver.find_element(By.ID, "warning_message")

    @property
    def next_run_button(self) -> WebElement:
        """Return the button for returning the next run."""
        return self.driver.find_element(By.ID, "next")

    @property
    def previous_run_button(self) -> WebElement:
        """Return the button for returning the previous run."""
        return self.driver.find_element(By.ID, "previous")

    @property
    def toggle_data_path_button(self) -> WebElement:
        """Return the toggle button for toggling the form on the page."""
        return self.driver.find_element(By.ID, "datapath_toggle")

    def run_description_text(self) -> str:
        """Return the text of the 'run_description' field."""
        return self.driver.find_element(By.ID, "run_description").text

    def title_text(self) -> str:
        """Return the text of the title field."""
        return self.driver.find_element(By.ID, "runTitle").text

    def started_by_text(self) -> str:
        """Return the text of the 'started_by' field."""
        return self.driver.find_element(By.ID, "started_by").text

    def status_text(self) -> str:
        """Return the text of the 'status' field."""
        return self.driver.find_element(By.ID, "status").text

    def instrument_text(self) -> str:
        """Return the text of the 'instrument' field."""
        return self.driver.find_element(By.ID, "instrument").text

    def rb_number_text(self) -> str:
        """Return the text of the 'rb_number' field."""
        return self.driver.find_element(By.ID, "rb_number").text

    def last_updated_text(self) -> str:
        """Return the text of the 'last_updated' field."""
        return self.driver.find_element(By.ID, "last_updated").text

    def data_path_text(self) -> str:
        """Return the text of the data location field."""
        return self.driver.find_element(By.CLASS_NAME, "file-path").text

    def reduction_host_text(self) -> WebElement:
        """Return the text of the 'reduction_host' field."""
        return self.driver.find_element(By.ID, "reduction_host").text

    def images(self) -> List[WebElement]:
        """Returns all image elements."""
        return self.driver.find_elements(By.TAG_NAME, "img")

    def plotly_plots(self) -> List[WebElement]:
        """Return all plotly plot elements."""
        return self.driver.find_elements(By.CLASS_NAME, "js-plotly-plot")

    def _do_cancel_btn(self, url):

        def run_button_clicked_successfully(button, url, driver):
            button.click()
            return driver.current_url.split("?")[0].endswith(url)

        button = self.driver.find_element(By.ID, "cancel")
        WebDriverWait(self.driver, 10).until(partial(run_button_clicked_successfully, button, url))

    def click_cancel_btn(self):
        """Click the cancel button and return a RunsListPage object."""
        # pylint:disable=cyclic-import,import-outside-toplevel
        from autoreduce_frontend.selenium_tests.pages.runs_list_page import RunsListPage

        self._do_cancel_btn(reverse("runs:list", kwargs={"instrument": self.instrument}))
        return RunsListPage(self.driver, self.instrument)

    def click_btn_by_id(self, btn_id: str) -> None:
        """Click the button with the given id."""
        btn = self.driver.find_element(By.ID, btn_id)
        btn.click()

    def is_disabled(self, element_id):
        """Return True if the element with the given id is disabled, otherwise, False."""
        return "disabled" in self.driver.find_element(By.ID, element_id).get_attribute("class")
